/*
 * GenericFuncs.c
 *
 * This file contains the library of addtional C functions, which
 * are generic enough to be used in any program
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>

#define BLOCKSIZE	512



int fputnchars(FILE *pstream, int n, unsigned char *pstring)
{
	int i;
	for(i=0; i<n; i++)
	{
		if(*(pstring+i))
			fputc(*(pstring+i), pstream);
		else
			break;
	}
	return i;
}



/*
 * The very simple: n-th power of x
 * (from Kernighan and Ritchie)
 */
int power(int x, int n)
{
	int i, p;
	if(!n)
		return 1;
	else
	{
		p = 1;
		for(i = 1; i <= n; ++i)
			p = p * x;
	}
	return(p);
}




/*
 * This function simply exchanges the upper- and lowercase of the character
 */
int characterCaseExchange(int character)
{
	if(isupper(character))
		character = tolower(character);
	else
	{
		if(islower(character))
			character = toupper(character);
	}
	return character;
}




/*
 * This function dumps the BLOCKSIZE number of bytes
 * to the stdout using NUMBEROFCOLUMNS bytes per line.
 * It returns number of bytes dumped.
 */
int hexDumpBlock(unsigned char *pblockpointer, int blocksize, int numberOfColumns)
{
	int i=0, j=numberOfColumns;
	fprintf(stdout, "%04x : ", i);
	for (i=0; i < BLOCKSIZE; i++)
	{
		if (!j)
		{
			fprintf(stdout, "\n");
			j=numberOfColumns;
			fprintf(stdout, "%04x : ", i);
		}
		fprintf(stdout, "%02x ", *(pblockpointer+i));
		j--;
	}
	fprintf(stdout, "\n");
	return i;
}



/*
 * nthByteFromFile()
 *
 * reads the byte number n from the given file
 * the return value is the read byte (success) or -1 (failure)
 */
static long int logicalblocklastread = -1;
static unsigned char blockbuffer[BLOCKSIZE];

int nthByteFromFile(FILE *pfilehandle, unsigned long int n)
{
	int debug = 0;
	unsigned char readbyte;
	int seekresult;
	long int logicalblocktoread;

	logicalblocktoread = n / BLOCKSIZE;
	if(logicalblocktoread == logicalblocklastread)
	{
		if(debug)
			fprintf(stderr, "nthByteFromFile() - same block [%ld], using cached... ", logicalblocklastread);
	}
	else
	{
		if(debug)
			fprintf(stderr, "nthByteFromFile() - seeking to byte: %ld (block containing the %ld)... ", (n / BLOCKSIZE) * BLOCKSIZE, n);

		seekresult = fseek(pfilehandle, (n / BLOCKSIZE) * BLOCKSIZE, SEEK_SET);

		if(debug)
			fprintf(stderr, "%ld... ", ftell(pfilehandle));

		if(seekresult)
		{
			fprintf(stderr, "nthByteFromFile() - seek result:\t%d\n", seekresult);
			perror("fseek");
			return -1;
		}
		else
		{
			if(fread(blockbuffer, BLOCKSIZE, 1, pfilehandle) != 1)
			{
				perror("fread");
				logicalblocklastread = -1;
				return -1;
			}
			else
			{
				logicalblocklastread = n / BLOCKSIZE;
			}
		}
	}
	readbyte = blockbuffer[n % BLOCKSIZE];
	if(debug)
		fprintf(stderr, "%02x\n", readbyte);
	return readbyte;
}


/*
 * writeNthByteTofile()
 *
 * writes the given byte to the given file at the given
 * position, using the given blocksize (e.g. for block device files)
 */
int writeNthByteToFile(FILE *pfilehandle, int blocksize, unsigned long int n, unsigned char bytetowrite)
{
	int seekresult;
	long int logicalblocktoread;

	logicalblocktoread = n / blocksize;
	
	if((seekresult = fseek(pfilehandle, logicalblocktoread * blocksize, SEEK_SET)))
	{
		fprintf(stderr, "writeNthByteToFile() - seek result:\t%d\n", seekresult);
		perror("fseek");
		return -1;
	}

	if(logicalblocktoread != logicalblocklastread)
	{
		if(fread(blockbuffer, blocksize, 1, pfilehandle) != 1)
		{
			perror("fread");
			logicalblocklastread = -1;
			return -1;
		}
		logicalblocklastread = logicalblocktoread;
		if((seekresult = fseek(pfilehandle, logicalblocktoread * blocksize, SEEK_SET)))
		{
			fprintf(stderr, "writeNthByteToFile() - seek result:\t%d\n", seekresult);
			perror("fseek");
			return -1;
		}
	}

	blockbuffer[n % blocksize] = bytetowrite;
	
	if(fwrite(blockbuffer, blocksize, 1, pfilehandle) < 1)
	{
		fprintf(stderr, "writeNthByteToFile() - ");
		perror("fwrite");
		logicalblocklastread = -1;
		return -1;
	}

	return 1;
}




// IMPROVEME: rewrite this as macro.
unsigned char decimal2BCD(unsigned char decimal)
{
	unsigned char bcd;
	return bcd = 0 | ((decimal / 10) << 4) | (decimal % 10);
}





char * disassemblePath(char *ppath, int component)
{
#define PATH	0
#define FILENAME	1
	char *ppathcomponent;

	// let's get rid of the leading white spaces (if any)
	while(*ppath == ' '
	   || *ppath == '\t')
		ppath++;
	
	// do the same to the trailing ones (if any)
	while((ppathcomponent = strrchr(ppath, ' ')) == (ppath + strlen(ppath) - 1)
	   || (ppathcomponent = strrchr(ppath, '\t')) == (ppath + strlen(ppath) - 1))
		*(ppath + strlen(ppath) -1) = '\0';

	if((ppathcomponent = strrchr(ppath, '/')))
	{
		if(component == PATH)
			*(ppathcomponent + 1) = '\0';

		if(component == FILENAME)
			ppath = ppathcomponent + 1;
	}
	else
	{
		if(component == PATH)
			strcpy(ppath, "./");
	}
	
	return ppath;
}




/*
 * stripExtension()
 *
 * Strips any .EXT from the given string
 * RETURNS: 0 on success, -1 on errors
 */
int stripExtension(unsigned char *pstring)
{
	int result = 0;
	char *pextension;

	if((pextension = strrchr(pstring, '.')))
		*(pextension) = '\0';
	else
		result = -1;

	return result;
}





