/*
 * SD_IDE64_Block.c
 *
 * This file holds the functions related
 * to block reads and writes to the file (harddrive)
 */

#include "SD_IDE64.h"


/*
 * SD_IDE64_BlockRead()
 *
 * RETURNS: number of blocks read (0 on error)
 */
int SD_IDE64_BlockRead(FILE * pfilehandle, struct chs *pblockschs, unsigned char * pblockbuffer)
{
	int debug = 0;
	unsigned long int logicalblocknumber;
	int numberofblocksread, numberofblockstoread = 1;
	int seekresult;

	// First we have to translate the chs into logical block
	logicalblocknumber = SD_IDE64_CHS2LogicalBlock(pblockschs);

	// Then we may start reading the blocks, provided it is within the drive's limits
	if(logicalblocknumber >= 0)
	{
		if(flags.debug && debug)
		{
			fprintf(stderr, "Trying to read block:\t %d/%d/%d\n", pblockschs->cylhi * 256 + pblockschs->cyllo, pblockschs->head, pblockschs->sec);
			fprintf(stderr, "Seeking to byte: %ld = (%ld*%d)\n", phddinfo->blocksize * logicalblocknumber, logicalblocknumber, phddinfo->blocksize);
		}
		seekresult = fseek(pfilehandle, phddinfo->blocksize * logicalblocknumber, SEEK_SET);
		if(flags.debug && debug)
		{
			fprintf(stderr, "Current position:\t%ld\n", ftell(pfilehandle));
		}
		if(seekresult)
		{
			fprintf(stderr, "Seek result:\t%d\n", seekresult);
			perror("fseek");
			return 0;
		}
	}
	else
	{
		return 0;
	}

	numberofblocksread = fread(pblockbuffer, phddinfo->blocksize, numberofblockstoread, pfilehandle);

	if(feof(pfilehandle) || ferror(pfilehandle))
	{
		fprintf(stderr, "SD_IDE64_BlockRead() - ");
		perror("fread");
	}

	if(flags.debug && debug)
		fprintf(stdout, "NumberOfbytesRead:\t%d\n", numberofblocksread * phddinfo->blocksize);

	return numberofblocksread;
}





/*
 * SD_IDE64_BlockWrite()
 *
 * Writes the block to the file given in first argument.
 * Block is addresed by CHS pointer. Content is taken from
 * the buffer pointed to by the last argument.
 * RETURNS: number of blocks written (1) - 0 on error
 */
int SD_IDE64_BlockWrite(FILE * pfilehandle, struct chs *pblockschs, unsigned char * pblockbuffer)
{
	int debug = 0;
	unsigned long int logicalblocknumber;
	int numberofblockswritten, numberofblockstowrite = 1;
	int seekresult;

	// First we have to translate the chs into logical block
	logicalblocknumber = SD_IDE64_CHS2LogicalBlock(pblockschs);

	// Then we may start writing the blocks, provided it is within the drive's limits
	if(logicalblocknumber >= 0)
	{
		if(flags.debug && debug)
		{
			fprintf(stdout, "Trying to write block:\t %d/%d/%d\n", pblockschs->cylhi * 256 + pblockschs->cyllo, pblockschs->head, pblockschs->sec);
			fprintf(stdout, "Seeking to byte: %ld = (%ld*%d)\n", phddinfo->blocksize * logicalblocknumber, logicalblocknumber, phddinfo->blocksize);
		}
		seekresult = fseek(pfilehandle, phddinfo->blocksize * logicalblocknumber, SEEK_SET);
		if(flags.debug && debug)
		{
			fprintf(stdout, "Current position:\t%ld\n", ftell(pfilehandle));
		}
		if(seekresult)
		{
			fprintf(stderr, "SD_IDE64_BlockWrite() - ");
			perror("fseek");
			return 0;
		}
	}
	else
	{
		fprintf(stderr, "SD_IDE64_BlockWrite() - Invalid block number?");
		return 0;
	}

	numberofblockswritten = fwrite(pblockbuffer, phddinfo->blocksize, numberofblockstowrite, pfilehandle);
	if(ferror(pfilehandle) || (numberofblockswritten < numberofblockstowrite))
	{
		fprintf(stderr, "SD_IDE64_BlockWrite() - ");
		perror("fwrite");
	}
	if(flags.debug && debug)
	{
		fprintf(stderr, "NumberOfbyteswritten:\t%d\n", numberofblockswritten * phddinfo->blocksize);
	}

	return numberofblockswritten;
}





/*
 * SD_IDE64_ReadFileBlock()
 *
 * Reads the file block starting at given CHS.
 * Stores the content at the location given by the pfileblock
 *
 * Returns:	 0 - if the read block was the last block of the file
 *			 1 - if the file has more blocks to read
 *			-1 - on error conditions
 */
int SD_IDE64_ReadFileBlock(struct chs *pchs, struct fileblock *pfileblock)
{
	int debug = 0;
	if(flags.debug && debug)
		fprintf(stdout, "Trying to read block [%d/%d/%d]... ", pchs->cylhi * 256 + pchs->cyllo, pchs->head, pchs->sec);

	if(SD_IDE64_BlockRead(phddinfo->pdevicefh, pchs, (unsigned char *)pfileblock))
	{
		if(flags.debug && debug)
		{
			fprintf(stdout, "OK, ");
		}

		if(pfileblock->nextfileblock.cylhi & 1<<6)
		{
			if(flags.debug && debug)
				fprintf(stdout, "The last one.\n");
			return 0;
		}
		else
		{
			if(flags.debug && debug)
				fprintf(stdout, "The next one is:\t[%d/%d/%d]\n", (pfileblock->nextfileblock.cylhi & CLEAR2MSBITSOF8) * 256 + pfileblock->nextfileblock.cyllo, pfileblock->nextfileblock.head, pfileblock->nextfileblock.sec);
			return 1;
		}
	}
	else
	{
		return -1;
	}
}




