/*
 * SD_IDE64_LoginDrive.c
 *
 * This file holds the function related to the global
 * drive information.
 */


#include "SD_IDE64.h"

/*
 * SD_IDE64_DriveLogin()
 *
 * Checks the given block device / disk image for the IDE64 format
 * and logs all the required information to the appropriate structures
 * RETURNS: 0 on success, -1 on errors
 */
int SD_IDE64_DriveLogin(const char *pdevicefile)
{
	struct chs rootchs, *prootchs = &rootchs;
	struct rootblock *prootblock;
	struct dirblock *pdirblock;
	int debug = 0;
	int result = 0;

	phddinfo = malloc(sizeof(struct hddinfo));
	if(phddinfo)
	{
		if(!(stat(pdevicefile, &phddinfo->sb)))
		{
			phddinfo->ppasswd = getpwuid(phddinfo->sb.st_uid);
			if(phddinfo->ppasswd)
			{
				phddinfo->pgroup = getgrgid(phddinfo->sb.st_gid);
				if(phddinfo->ppasswd)
				{
					if(debug)
						fprintf(stdout, "%s\t%s\t%s\t%d\t%d\n", bb_mode_string(phddinfo->sb.st_mode), phddinfo->ppasswd->pw_name, phddinfo->pgroup->gr_name, (int)phddinfo->sb.st_blocks, (int)phddinfo->sb.st_blksize);

					if(flags.verbose)
						printf("Opening HDD file...");

					if(flags.readonly)
						phddinfo->pdevicefh = fopen(pdevicefile, "r");
					else
						phddinfo->pdevicefh = fopen(pdevicefile, "r+");

					if(phddinfo->pdevicefh)
					{
						if(flags.verbose)
							printf("OK\n");

						prootblock = malloc(sizeof(struct rootblock));
						if(prootblock)
						{
							// These are the initial geometry values. We use those because we still don't know
							// the real (stored) values. It makes no harm since the root block is anyway
							// located on 0/0/1 - being always the first block, regardless of the geometry.
							// Once we read the root block, we fill the proper values in.
							phddinfo->geometry.cylhi = 0;
							phddinfo->geometry.cyllo = 1;
							phddinfo->geometry.head = 1;
							phddinfo->geometry.sec = 1;

							prootchs->cylhi = ROOTCYL / 256;
							prootchs->cyllo = ROOTCYL % 256;
							prootchs->head = ROOTHEAD;
							prootchs->sec = ROOTSEC;
							// IMPROVEME: the real BLOCKSIZE of the drive has to be read-in, not assumed
							phddinfo->blocksize = BLOCKSIZE;
							if(debug)
								fprintf(stderr, "blocksize %d\n", phddinfo->blocksize);

							if(flags.verbose)
								fprintf(stdout, "Reading info block [%d/%d/%d]... ",prootchs->cylhi * 256 + prootchs->cyllo, prootchs->head, prootchs->sec);

							if(SD_IDE64_BlockRead(phddinfo->pdevicefh, prootchs, (unsigned char *)prootblock))
							{
								if(flags.verbose)
									fprintf(stdout, "OK\nSearching for signature... ");
								//IMPROVEME: this should be placed earlier and scan the available block devices if none was specified
								if(strncmp(prootblock->identification, ide64signature, sizeof(ide64signature)))
								{
									// notify here to format the drive first?
									fprintf(stderr, "SD_IDE64_DriveLogin() - Warning: IDE64 signature not found!\n");
								}
								else
								{
									memcpy(&phddinfo->identification, prootblock->identification, sizeof(ide64signature));
									if(flags.verbose)
										fprintf(stdout, "OK\nReading drive information... ");
								}

								phddinfo->geometry.cylhi = (prootblock->lastdriveblock.cylhi * 256 + prootblock->lastdriveblock.cyllo + 1) / 256;
								phddinfo->geometry.cyllo = (prootblock->lastdriveblock.cylhi * 256 + prootblock->lastdriveblock.cyllo + 1) % 256;
								phddinfo->geometry.head = prootblock->lastdriveblock.head + 1;
								phddinfo->geometry.sec = prootblock->lastdriveblock.sec;
								phddinfo->numberoflogicalblocks = (phddinfo->geometry.cylhi * 256 + phddinfo->geometry.cyllo) * phddinfo->geometry.head * phddinfo->geometry.sec;

								memcpy(&phddinfo->rootdirchs, &prootblock->rootdirblock, sizeof(struct chs));
								memcpy(&phddinfo->bam1start, &prootblock->bam1block, sizeof(struct chs));
								memcpy(&phddinfo->bam2start, &prootblock->bam2block, sizeof(struct chs));

								pdirblock = malloc(sizeof(struct dirblock));
								if(pdirblock)
								{
									if(SD_IDE64_BlockRead(phddinfo->pdevicefh, &phddinfo->rootdirchs, (unsigned char *)pdirblock))
									{
										memcpy(&phddinfo->root_direntry, &pdirblock->direntry[0], sizeof(struct direntry));
										if(flags.verbose)
											fprintf(stdout, "OK\nGetting root directory... ");
										// At the end let's change the current dir to the root (cd /)
										if(!SD_IDE64_CDPath("/"))
										{
											if(flags.verbose)
												fprintf(stdout, "OK\n");
											if(flags.debug)
											{
												fprintf(stdout, "done.\n");
												SD_IDE64_PrintDriveInfo(phddinfo);
											}
										}
										else
										{
											fprintf(stderr, "SD_IDE64_DriveLogin() - error changing to root dir!\n");
											result = -1;
										}
									}
									else
									{
										fprintf(stderr, "SD_IDE64_DriveLogin() - error reading the root directory block!\n");
										result = -1;
									}
									free(pdirblock);
								}
								else
								{
									fprintf(stderr, "SD_IDE64_DriveLogin() - ");
									perror("malloc");
									result = -1;
								}
							}
							else
							{
								fprintf(stderr, "SD_IDE64_DriveLogin() - error reading block [%d/%d/%d]\n", ROOTCYL, ROOTHEAD, ROOTSEC);
								result = -1;
							}
							free(prootblock);
						}
						else
						{
							fprintf(stderr, "SD_IDE64_DriveLogin() - ");
							perror("malloc");
							result = -1;
						}

					}
					else
					{
						if(flags.verbose)
							printf("Bad luck!\n");
						fprintf(stderr, "SD_IDE64_DriveLogin() - ");
						perror("fopen");
						result = -1;
					}
				}
				else
				{
					fprintf(stderr, "SD_IDE64_DriveLogin() - ");
					perror("getgrgid");
					result = -1;
				}
			}
			else
			{
				fprintf(stderr, "SD_IDE64_DriveLogin() - ");
				perror("getpwuid");
				result = -1;
			}
		}
		else
		{
			fprintf(stderr, "SD_IDE64_DriveLogin() - ");
			perror("stat");
			result = -1;
		}
	}
	else
	{
		fprintf(stderr, "SD_IDE64_DriveLogin() - ");
		perror("malloc");
		result = -1;
	}
	return result;
}





/*
 * SD_IDE64_DriveLogout()
 *
 * Performs a "log-out" of the drive, which includes flushing
 * the BAM cache, freeing the info structure and closing the device file.
 * Something more may come in the future
 * RETURNS: 0 on success, -1 on error
 */
int SD_IDE64_DriveLogout(void)
{
	int result = 0;
	if(!(flags.readonly))
	{
		if(SD_IDE64_BAMByte(0, BAM_FLUSH, 0))
		{
			fprintf(stderr, "SD_IDE64_DriveLogout() - error flushing the BAM cache.\n");
			result = -1;
		}
	}
	if(flags.verbose)
		printf("Closing HDD file... ");
	if(fclose(phddinfo->pdevicefh))
	{
		if(flags.verbose)
			printf("ERROR!\n");
		fprintf(stderr, "SD_IDE64_DriveLogout() - ");
		perror("fclose");
		result = -1;
	}
	else
	{
		if(flags.verbose)
			printf("OK.\n");
	}
	free(phddinfo);
	return result;
}
